from logging.config import fileConfig
import os, sys
from dotenv import load_dotenv

# from sqlalchemy import engine_from_config
from sqlalchemy import create_engine

from alembic import context

alembic_dir = os.path.dirname(os.path.realpath(__file__))
# logger.info(f"Got alembic_dir: {alembic_dir}")
sys.path.append(alembic_dir)
load_dotenv(os.path.join(alembic_dir, ".env"))


# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)

# add your model's MetaData object here
# for 'autogenerate' support
# from ceonmedia_db.database import Base

# from ceonmedia_db.models.user_models import *
# from ceonmedia_db.models.project_models import *
# from ceonmedia_db.models import *
from ceonmedia_db.models.base import Base

target_metadata = Base.metadata
# target_metadata = None


# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.


def get_url():
    DB_URI = os.getenv("DB_URI")
    # logger.info(f"Alembic got DB_URI: {DB_URI}")
    if not DB_URI:
        raise Exception("Environment variable DB_URI not set")
    return DB_URI


def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    # url = config.get_main_option("sqlalchemy.url")
    url = get_url()
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    # connectable = engine_from_config(
    # config.get_section(config.config_ini_section),
    # prefix="sqlalchemy.",
    # poolclass=pool.NullPool,
    # )
    connectable = create_engine(get_url())

    with connectable.connect() as connection:
        context.configure(connection=connection, target_metadata=target_metadata)

        with context.begin_transaction():
            context.run_migrations()


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()
